#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

# import standard library

# import interfaces
from pyams_skin.interfaces.viewlet import IToolbarViewletManager
from pyams_workflow.interfaces import IWorkflow, IWorkflowInfo, IWorkflowVersion, IWorkflowManagedContent
from pyams_workflow.zmi.interfaces import IWorkflowMenu
from pyams_zmi.layer import IAdminLayer

# import packages
from pyams_skin.viewlet.toolbar import ToolbarMenu, ToolbarMenuItem
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from zope.interface import implementer, Interface

from pyams_workflow import _


class WorkflowMenuItem(ToolbarMenuItem):
    """Workflow menu item"""

    def __init__(self, context, request, view, manager, transition):
        super(WorkflowMenuItem, self).__init__(context, request, view, manager)
        self.label = transition.title
        self.label_css_class = transition.user_data.get('menu_css_class') or 'fa fa-fw'
        self.url = '{url}?form.widgets.transition_id={transition}'.format(url=transition.user_data.get('view_name'),
                                                                          transition=transition.transition_id)
        self.weight = transition.order
        self.modal_target = True


@viewlet_config(name='workflow.menu', context=IWorkflowVersion, layer=IAdminLayer,
                view=Interface, manager=IToolbarViewletManager, weight=800)
@viewletmanager_config(name='workflow.menu', layer=IAdminLayer, provides=IWorkflowMenu)
@implementer(IWorkflowMenu)
class WorkflowMenu(ToolbarMenu):
    """Workflow menu"""

    label = _("Change status...")
    label_css_class = ''
    css_class = 'btn btn-xs btn-danger dropdown-toggle'

    def _get_viewlets(self):
        viewlets = []
        parent = get_parent(self.context, IWorkflowManagedContent)
        wf = query_utility(IWorkflow, name=parent.workflow_name)
        if wf is None:
            return viewlets
        info = IWorkflowInfo(self.context)
        for transition_id in info.get_manual_transition_ids():
            transition = wf.get_transition_by_id(transition_id)
            menu = WorkflowMenuItem(self.context, self.request, self.__parent__, self, transition)
            viewlets.append((transition_id, menu))
        viewlets.sort(key=lambda x: x[1].weight)
        return viewlets
